@@ -27,6 +27,7 @@
 
 import os
 import sys
+import socket
 
 DEFAULT_PARALLELISM = 32
 DEFAULT_TIMEOUT = 0  # "infinity" by default
@@ -54,14 +55,14 @@ def to_ascii(s):
         return s
 
 
-class Error(BaseException):
+class Error(Exception):
     """
     Returned instead of a result for a host
     in case of an error during the processing for
     that host.
     """
     def __init__(self, msg, task):
-        super(BaseException, self).__init__()
+        super(Exception, self).__init__()
         self.msg = msg
         self.task = task
 
@@ -87,6 +88,7 @@ class Options(object):
     askpass = False              # Ask for a password
     outdir = None                # Write stdout to a file per host in this directory
     errdir = None                # Write stderr to a file per host in this directory
+    ssh_key = None               # Specific ssh key used by ssh/scp -i option
     ssh_options = []             # Extra options to pass to SSH
     ssh_extra = []               # Extra arguments to pass to SSH
     verbose = False              # Warning and diagnostic messages
@@ -138,19 +140,21 @@ class _CallOutputBuilder(object):
         return ret
 
 
-def _build_call_cmd(host, port, user, cmdline, options, extra):
+def _build_call_cmd(host, port, user, cmdline, opts):
     cmd = ['ssh', host,
            '-o', 'NumberOfPasswordPrompts=1',
            '-o', 'SendEnv=PARALLAX_NODENUM PARALLAX_HOST']
-    if options:
-        for opt in options:
+    if opts.ssh_options:
+        for opt in opts.ssh_options:
             cmd += ['-o', opt]
     if user:
         cmd += ['-l', user]
     if port:
         cmd += ['-p', port]
-    if extra:
-        cmd.extend(extra)
+    if opts.ssh_key:
+        cmd += ['-i', opts.ssh_key]
+    if opts.ssh_extra:
+        cmd.extend(opts.ssh_extra)
     if cmdline:
         cmd.append(cmdline)
     return cmd
@@ -158,7 +162,7 @@ def _build_call_cmd(host, port, user, cmdline, options, extra):
 
 def call(hosts, cmdline, opts=Options()):
     """
-    Executes the given command on a set of hosts, collecting the output
+    Executes the given command on a set of hosts, collecting the output. Return Error when exit status != 0.
     Returns {host: (rc, stdout, stdin) | Error}
     """
     if opts.outdir and not os.path.exists(opts.outdir):
@@ -173,9 +177,11 @@ def call(hosts, cmdline, opts=Options()):
                       warn_message=opts.warn_message,
                       callbacks=_CallOutputBuilder())
     for host, port, user in _expand_host_port_user(hosts):
-        cmd = _build_call_cmd(host, port, user, cmdline,
-                              options=opts.ssh_options,
-                              extra=opts.ssh_extra)
+        is_local = is_local_host(host)
+        if is_local:
+            cmd = [cmdline]
+        else:
+            cmd = _build_call_cmd(host, port, user, cmdline, opts)
         t = Task(host, port, user, cmd,
                  stdin=opts.input_stream,
                  verbose=opts.verbose,
@@ -183,7 +189,8 @@ def call(hosts, cmdline, opts=Options()):
                  print_out=opts.print_out,
                  inline=opts.inline,
                  inline_stdout=opts.inline_stdout,
-                 default_user=opts.default_user)
+                 default_user=opts.default_user,
+                 is_local=is_local)
         manager.add_task(t)
     try:
         return manager.run()
@@ -219,13 +226,15 @@ def _build_copy_cmd(host, port, user, src, dst, opts):
         cmd += ['-P', port]
     if opts.recursive:
         cmd.append('-r')
+    if opts.ssh_key:
+        cmd += ['-i', opts.ssh_key]
     if opts.ssh_extra:
         cmd.extend(opts.ssh_extra)
     cmd.append(src)
     if user:
-        cmd.append('%s@%s:%s' % (user, host, dst))
+        cmd.append('%s@[%s]:%s' % (user, host, dst))
     else:
-        cmd.append('%s:%s' % (host, dst))
+        cmd.append('[%s]:%s' % (host, dst))
     return cmd
 
 
@@ -312,12 +321,14 @@ def _build_slurp_cmd(host, port, user, src, dst, opts):
         cmd += ['-P', port]
     if opts.recursive:
         cmd.append('-r')
+    if opts.ssh_key:
+        cmd += ['-i', opts.ssh_key]
     if opts.ssh_extra:
         cmd.extend(opts.ssh_extra)
     if user:
-        cmd.append('%s@%s:%s' % (user, host, src))
+        cmd.append('%s@[%s]:%s' % (user, host, src))
     else:
-        cmd.append('%s:%s' % (host, src))
+        cmd.append('[%s]:%s' % (host, src))
     cmd.append(dst)
     return cmd
 
@@ -361,3 +372,73 @@ def slurp(hosts, src, dst, opts=Options()):
         return manager.run()
     except FatalError as err:
         raise IOError(str(err))
+
+
+def is_local_host(host):
+    """
+    Check if the host is local
+    """
+    try:
+        socket.inet_aton(host)
+        hostname = socket.gethostbyaddr(host)[0]
+    except:
+        hostname = host
+    return hostname == socket.gethostname()
+
+def run(hosts, cmdline, opts=Options()):
+    """
+    Executes the given command on a set of hosts, collecting the output. Return Error when ssh error occurred.
+    Returns {host: (rc, stdout, stdin) | Error}
+    """
+    if opts.outdir and not os.path.exists(opts.outdir):
+        os.makedirs(opts.outdir)
+    if opts.errdir and not os.path.exists(opts.errdir):
+        os.makedirs(opts.errdir)
+    manager = Manager(limit=opts.limit,
+                      timeout=opts.timeout,
+                      askpass=opts.askpass,
+                      outdir=opts.outdir,
+                      errdir=opts.errdir,
+                      warn_message=opts.warn_message,
+                      callbacks=_RunOutputBuilder())
+    for host, port, user in _expand_host_port_user(hosts):
+        is_local = is_local_host(host)
+        if is_local:
+            cmd = [cmdline]
+        else:
+            cmd = _build_call_cmd(host, port, user, cmdline, opts)
+        t = Task(host, port, user, cmd,
+                 stdin=opts.input_stream,
+                 verbose=opts.verbose,
+                 quiet=opts.quiet,
+                 print_out=opts.print_out,
+                 inline=opts.inline,
+                 inline_stdout=opts.inline_stdout,
+                 default_user=opts.default_user,
+                 is_local=is_local)
+        manager.add_task(t)
+    try:
+        return manager.run()
+    except FatalError as err:
+        raise IOError(str(err))
+
+
+class _RunOutputBuilder(object):
+    def __init__(self):
+        self.finished_tasks = []
+
+    def finished(self, task, n):
+        """Called when Task is complete"""
+        self.finished_tasks.append(task)
+
+    def result(self, manager):
+        """Called when all Tasks are complete to generate result"""
+        ret = {}
+        for task in self.finished_tasks:
+            if task.exitstatus == 255:
+                ret[task.host] = Error(', '.join(task.failures), task)
+            else:
+                ret[task.host] = (task.exitstatus,
+                                  task.outputbuffer or manager.outdir,
+                                  task.errorbuffer or manager.errdir)
+        return ret
